## Topics of this Example

This sample reproduced OOP inheritance with mixin.
Inherits Todos used in other examples.
The subclass deals with display interest as a presentation layer.

```javascript
import { TodosST, TodosQR, TodosMT, TodosModel } from './todos'
import { TodoST } from './todo'

// ______________________________________________________
//
// @ Model

export interface TodosPresentST extends TodosST {
  showAll: boolean
}
export const TodosPresentModel: Modeler<TodosPresentST> = injects => ({
  ...TodosModel(),
  showAll: true,
  ...injects
})

// ______________________________________________________
//
// @ Queries

function getDoingItems (state: TodosPresentST): TodoST[] {
  return state.items.filter(item => !item.done)
}
function getDoneItems (state: TodosPresentST): TodoST[] {
  return state.items.filter(item => item.done)
}
function getVisibleItems (state: TodosPresentST): TodoST[] {
  return state.showAll ? state.items : getDoingItems(state)
}
function getTodosCountStatusLabel (state: TodosPresentST): string {
  const all = `all: ${state.items.length}`
  const doing = `doing: ${getDoingItems(state).length}`
  const done = `done: ${getDoneItems(state).length}`
  return `${all} / ${doing} / ${done}`
}
function getToggleVisibleItemsBtnLabel (state: TodosPresentST): string {
  return state.showAll ? 'hide done items' : 'show all items'
}
export const TodosPresentQR = {
  ...TodosQR,
  getVisibleItems,
  getTodosCountStatusLabel,
  getToggleVisibleItemsBtnLabel
}
```

With use [immer](https://www.npmjs.com/package/immer), immutable change become easily.
Use is optional, but there is no doubt that you can concentrate more on development.

```javascript
// ______________________________________________________
//
// @ Mutations

const toggleShowAll = (
  state: TodosPresentST
): TodosPresentST => immer(state, _state => {
  _state.showAll = !_state.showAll
})
export const TodosPresentMT = {
  ...TodosMT,
  toggleShowAll
}

```

